@@ -605,10 +605,17 @@ class ArrayNameSpace:
 
         """
 
-    def explode(self) -> Series:
+    def explode(self, *, empty_as_null: bool = True, keep_nulls: bool = True) -> Series:
         """
         Returns a column with a separate row for every array element.
 
+        Parameters
+        ----------
+        empty_as_null
+            Explode an empty array into a `null`.
+        keep_nulls
+            Explode a `null` array into a `null`.
+
         Returns
         -------
         Series
